"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.graphViewer = exports.getNodeById = exports.hideNodes = exports.revealNodes = exports.queryType = exports.getVisibleChildren = exports.getVisibleNodes = exports.graphBuilder = exports.removeNode = exports.addNode = exports.createNode = exports.removeConnection = exports.addConnection = exports.removeConnectionType = exports.addConnectionType = exports.removeNodeType = exports.updateNodeType = exports.addNodeType = exports.initGraph = void 0;
var error_1 = require("./error");
var compose = function (f) {
    return {
        f: function (g) { return compose(function (x) { return g(f(x)); }); },
        out: function () { return f; },
    };
};
var initGraph = function () {
    return {
        nodeSchema: [],
        graphSchema: { connections: [] },
        nodes: [],
        adjacencyList: {},
    };
};
exports.initGraph = initGraph;
var addNodeType = function (graph) { return function (schema) {
    var newType = schema.type;
    var exists = graph.nodeSchema.some(function (t) { return t.type === newType; });
    if (exists)
        throw new error_1.GraphError("Type of same name already exists");
    return __assign(__assign({}, graph), { nodeSchema: graph.nodeSchema.concat(schema) });
}; };
exports.addNodeType = addNodeType;
var updateNodeType = function (graph) { return function (schema) {
    var newType = schema.type;
    var exists = graph.nodeSchema.some(function (t) { return t.type === newType; });
    if (!exists)
        throw new error_1.GraphError("Type does not exist");
    return __assign(__assign({}, graph), { nodeSchema: graph.nodeSchema.map(function (t) { return (t.type === newType ? schema : t); }) });
}; };
exports.updateNodeType = updateNodeType;
var removeNodeType = function (graph) { return function (type) {
    var exists = graph.nodeSchema.some(function (t) { return t.type === type; });
    if (!exists)
        throw new error_1.GraphError("Type does not exist");
    return __assign(__assign({}, graph), { nodeSchema: graph.nodeSchema.filter(function (t) { return t.type !== type; }) });
}; };
exports.removeNodeType = removeNodeType;
var addConnectionType = function (graph) { return function (connection) {
    var validTypes = new Set(graph.nodeSchema.map(function (t) { return t.type; }));
    if (!connection.every(function (t) { return validTypes.has(t); }))
        throw new error_1.GraphError("Type does not exist");
    if (graph.graphSchema.connections.some(function (c) { return c[0] === connection[0] && c[1] === connection[1]; }))
        throw new error_1.GraphError("Connection type already exists");
    return __assign(__assign({}, graph), { graphSchema: __assign(__assign({}, graph.graphSchema), { connections: __spreadArray(__spreadArray([], __read(graph.graphSchema.connections), false), [connection], false) }) });
}; };
exports.addConnectionType = addConnectionType;
var removeConnectionType = function (graph) { return function (connection) {
    return __assign(__assign({}, graph), { graphSchema: __assign(__assign({}, graph.graphSchema), { connections: graph.graphSchema.connections.filter(function (c) { return c[0] !== connection[0] && c[1] !== connection[1]; }) }) });
}; };
exports.removeConnectionType = removeConnectionType;
var addConnection = function (graph) { return function (connection) {
    var _a;
    var _b;
    var existingIds = new Set(graph.nodes.map(function (n) { return n.id; }));
    var valid = connection.every(function (id) { return existingIds.has(id); });
    if (!valid)
        throw new error_1.GraphError("ID does not exist");
    var conIds = new Set(connection);
    var conTypes = graph.nodes
        .filter(function (n) { return conIds.has(n.id); })
        .map(function (n) { return n.type; });
    var validType = graph.graphSchema.connections.some(function (_a) {
        var _b = __read(_a, 2), l = _b[0], r = _b[1];
        return l === conTypes[0] && r === conTypes[1];
    });
    if (!validType)
        throw new error_1.GraphError("invalid connection type: ".concat(conTypes.join(" -> ")));
    var list = ((_b = graph.adjacencyList[connection[0]]) === null || _b === void 0 ? void 0 : _b.slice()) || [];
    if (list.indexOf(connection[0]) === -1)
        list.push(connection[1]);
    var adjacencyList = __assign(__assign({}, graph.adjacencyList), (_a = {}, _a[connection[0]] = list, _a));
    return __assign(__assign({}, graph), { adjacencyList: adjacencyList });
}; };
exports.addConnection = addConnection;
var removeConnection = function (graph) { return function (connection) {
    var existingIds = new Set(graph.nodes.map(function (n) { return n.id; }));
    var valid = connection.every(function (id) { return existingIds.has(id); });
    if (!valid)
        throw new error_1.GraphError("ID does not exist");
    var adjacencyList = Object.entries(graph.adjacencyList).reduce(function (a, _a) {
        var _b = __read(_a, 2), k = _b[0], v = _b[1];
        if (k === connection[0])
            a[k] = v.filter(function (id) { return id !== connection[1]; });
        else
            a[k] = __spreadArray([], __read(v), false);
        return a;
    }, {});
    return __assign(__assign({}, graph), { adjacencyList: adjacencyList });
}; };
exports.removeConnection = removeConnection;
var createNode = function (node) {
    return __assign({ id: Math.floor(Math.random() * Math.pow(36, 6)).toString(36) }, node);
};
exports.createNode = createNode;
var addNode = function (graph) { return function (node) {
    var existingIds = new Set(graph.nodes.map(function (n) { return n.id; }));
    if (existingIds.has(node.id))
        throw new error_1.GraphError("Id already exists");
    var existingTypes = new Set(graph.nodeSchema.map(function (t) { return t.type; }));
    if (!existingTypes.has(node.type))
        throw new error_1.GraphError("invalid type");
    return __assign(__assign({}, graph), { nodes: graph.nodes.concat(node) });
}; };
exports.addNode = addNode;
var removeNode = function (graph) {
    return function (id) {
        var adjacencyList = Object.entries(graph.adjacencyList).reduce(function (a, _a) {
            var _b = __read(_a, 2), k = _b[0], v = _b[1];
            if (k === id)
                return a;
            a[k] = v.filter(function (vId) { return vId !== id; });
            return a;
        }, {});
        return __assign(__assign({}, graph), { adjacencyList: adjacencyList, nodes: graph.nodes.filter(function (n) { return n.id !== id; }) });
    };
};
exports.removeNode = removeNode;
var graphBuilder = function (graph) {
    if (graph === void 0) { graph = (0, exports.initGraph)(); }
    return ({
        build: function () { return graph; },
        addNode: compose((0, exports.addNode)(graph)).f(exports.graphBuilder).out(),
        removeNode: compose((0, exports.removeNode)(graph)).f(exports.graphBuilder).out(),
        addConnection: compose((0, exports.addConnection)(graph)).f(exports.graphBuilder).out(),
        removeConnection: compose((0, exports.removeConnection)(graph)).f(exports.graphBuilder).out(),
        addConnectionType: compose((0, exports.addConnectionType)(graph)).f(exports.graphBuilder).out(),
        removeConnectionType: compose((0, exports.removeConnectionType)(graph))
            .f(exports.graphBuilder)
            .out(),
        addNodeType: compose((0, exports.addNodeType)(graph)).f(exports.graphBuilder).out(),
        updateNodeType: compose((0, exports.updateNodeType)(graph)).f(exports.graphBuilder).out(),
        removeNodeType: compose((0, exports.removeNodeType)(graph)).f(exports.graphBuilder).out(),
    });
};
exports.graphBuilder = graphBuilder;
var getVisibleNodes = function (graph) { return function (revealed) {
    var visible = new Set(revealed);
    revealed.forEach(function (id) {
        var children = graph.adjacencyList[id];
        if (!children)
            return;
        children.forEach(function (cId) { return visible.add(cId); });
    });
    return __spreadArray([], __read(visible), false);
}; };
exports.getVisibleNodes = getVisibleNodes;
var getVisibleChildren = function (graph, visible) { return function (id) {
    var vSet = new Set(visible);
    var vChildren = new Set((graph.adjacencyList[id] || []).filter(function (cId) { return vSet.has(cId); }));
    return graph.nodes.filter(function (n) { return vChildren.has(n.id); });
}; };
exports.getVisibleChildren = getVisibleChildren;
var queryType = function (nodes, type) {
    return nodes.filter(function (n) { return n.type === type; });
};
exports.queryType = queryType;
var revealNodes = function (viewerState) { return function (ids) {
    var rSet = new Set(viewerState.revealed);
    ids.forEach(function (x) { return rSet.add(x); });
    return { revealed: __spreadArray([], __read(rSet), false) };
}; };
exports.revealNodes = revealNodes;
var hideNodes = function (viewerState) { return function (ids) {
    var rSet = new Set(viewerState.revealed);
    ids.forEach(function (x) { return rSet.delete(x); });
    return { revealed: __spreadArray([], __read(rSet), false) };
}; };
exports.hideNodes = hideNodes;
var getNodeById = function (nodes) {
    var nodeMap = nodes.reduce(function (a, n) {
        a[n.id] = n;
        return a;
    }, {});
    return function (id) { return nodeMap[id]; };
};
exports.getNodeById = getNodeById;
var graphViewer = function (graph) {
    return function (viewerState) {
        var getVN = (0, exports.getVisibleNodes)(graph);
        var internalState = {
            revealed: __spreadArray([], __read(((viewerState === null || viewerState === void 0 ? void 0 : viewerState.revealed) || [])), false),
        };
        var visibleNodeIds = getVN(internalState.revealed);
        var vSet = new Set(visibleNodeIds);
        var vNodes = graph.nodes.filter(function (n) { return vSet.has(n.id); });
        var getVC = (0, exports.getVisibleChildren)(graph, visibleNodeIds);
        return {
            revealNodes: compose((0, exports.revealNodes)(internalState))
                .f((0, exports.graphViewer)(graph))
                .out(),
            hideNodes: compose((0, exports.hideNodes)(internalState)).f((0, exports.graphViewer)(graph)).out(),
            getTypes: function () {
                return graph.nodeSchema.map(function (s) { return ({
                    type: s.type,
                    fields: s.fields.map(function (f) { return (__assign({}, f)); }),
                }); });
            },
            getConnectionTypes: function () {
                return graph.graphSchema.connections.map(function (c) { return c.slice(); });
            },
            getChildren: getVC,
            getChildrenByType: function (id, type) {
                return (0, exports.queryType)(getVC(id), type);
            },
            getNodes: function () { return __spreadArray([], __read(graph.nodes), false); },
            getVNodes: function () { return __spreadArray([], __read(vNodes), false); },
            getByType: function (type) { return (0, exports.queryType)(vNodes, type); },
            getNodeById: (0, exports.getNodeById)(graph.nodes),
            getVNodeById: (0, exports.getNodeById)(vNodes),
            state: function () { return internalState; },
        };
    };
};
exports.graphViewer = graphViewer;
__exportStar(require("./error"), exports);
__exportStar(require("./types"), exports);
